// Main entry point for random map script
// Origional map by ES
// Adapted to work by petard_rusher

void main(void)
{
   // Set size.
   int playerTiles=12000;
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Set up default water.
   rmSetSeaLevel(0.0);
   rmSetSeaType("aegean sea");

   // Init map.
   rmTerrainInitialize("water");

   // Define some classes.
   int classPlayer=rmDefineClass("player");
   rmDefineClass("starting settlement");

   
   // Create a edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));

   // Player area constraint.
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 10.0);

   // Settlement constraint.
   int avoidSettlement=rmCreateTypeDistanceConstraint("avoid settlement", "AbstractSettlement", 50.0);
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("short avoid settlement", "AbstractSettlement", 10.0);

   // Far starting settlement constraint.
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("far start settle", rmClassID("starting settlement"), 50.0);

   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);
   int avoidTower2=rmCreateTypeDistanceConstraint("avoid tower2", "tower", 25.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // Goats/pigs
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 5.0);

   // Avoid food.
   int avoidFood=rmCreateTypeDistanceConstraint("avoid food", "food", 20.0);

   // Define some objects
   int startingSettlementID=rmCreateObjectDef("Starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 23.0);
   rmSetObjectDefMaxDistance(startingTowerID, 26.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);

   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 34.0);
   rmSetObjectDefMaxDistance(startingGoldID, 40.0);
   rmAddObjectDefConstraint(startingGoldID, avoidTower2);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);

   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 50.0);
   rmSetObjectDefMaxDistance(farGoldID, 100.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farGoldID, edgeConstraint);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);

   // int closePigsID=rmCreateObjectDef("close pigs");
   // rmAddObjectDefItem(closePigsID, "pig", 4, 6.0);
   // rmSetObjectDefMinDistance(closePigsID, 20.0);
   // rmSetObjectDefMaxDistance(closePigsID, 28.0);
   // rmAddObjectDefConstraint(closePigsID, avoidImpassableLand);

   int mediumPigsID=rmCreateObjectDef("medium pigs");
   rmAddObjectDefItem(mediumPigsID, "pig", 2, 4.0);
   rmSetObjectDefMinDistance(mediumPigsID, 50.0);
   rmSetObjectDefMaxDistance(mediumPigsID, 70.0);
   rmAddObjectDefConstraint(mediumPigsID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumPigsID, avoidFood);

   int farPigsID=rmCreateObjectDef("far pigs");
   rmAddObjectDefItem(farPigsID, "pig", 2, 4.0);
   rmSetObjectDefMinDistance(farPigsID, 50.0);
   rmSetObjectDefMaxDistance(farPigsID, 100.0);
   rmAddObjectDefConstraint(farPigsID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPigsID, avoidImpassableLand);
   rmAddObjectDefConstraint(farPigsID, avoidFood);

   int farBearsID=rmCreateObjectDef("far bears");
   rmAddObjectDefItem(farBearsID, "bear", 1, 0.0);
   rmSetObjectDefMinDistance(farBearsID, 50.0);
   rmSetObjectDefMaxDistance(farBearsID, 100.0);
   rmAddObjectDefConstraint(farBearsID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farBearsID, shortAvoidImpassableLand);

   int farCrocsID=rmCreateObjectDef("far Crocs");
   rmAddObjectDefItem(farCrocsID, "crocodile", 1, 0.0);
   rmSetObjectDefMinDistance(farCrocsID, 50.0);
   rmSetObjectDefMaxDistance(farCrocsID, 100.0);
   rmAddObjectDefConstraint(farCrocsID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farCrocsID, shortAvoidImpassableLand);

   // For this map, pick how many deer/gazelle in a grouping.  Assign this
   // to both deer and gazelle since we place them interchangeably per player.
   int numHuntable=rmRandInt(6, 10);

   int mediumGazelleID=rmCreateObjectDef("medium gazelles");
   rmAddObjectDefItem(mediumGazelleID, "gazelle", numHuntable, 5.0);
   rmSetObjectDefMinDistance(mediumGazelleID, 40.0);
   rmSetObjectDefMaxDistance(mediumGazelleID, 80.0);
   rmAddObjectDefConstraint(mediumGazelleID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGazelleID, avoidFood);

   int mediumDeerID=rmCreateObjectDef("medium deer");
   rmAddObjectDefItem(mediumDeerID, "deer", numHuntable, 5.0);
   rmSetObjectDefMinDistance(mediumDeerID, 40.0);
   rmSetObjectDefMaxDistance(mediumDeerID, 80.0);
   rmAddObjectDefConstraint(mediumDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumDeerID, avoidFood);

   int closeChickensID=rmCreateObjectDef("close Chickens");
   rmAddObjectDefItem(closeChickensID, "chicken", 8, 5.0);
   rmSetObjectDefMinDistance(closeChickensID, 20.0);
   rmSetObjectDefMaxDistance(closeChickensID, 25.0);
   rmAddObjectDefConstraint(closeChickensID, avoidFood);
   rmAddObjectDefConstraint(closeChickensID, avoidImpassableLand);

   int closeBerriesID=rmCreateObjectDef("close berries");
   rmAddObjectDefItem(closeBerriesID, "berry bush", 6, 4.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);
   rmAddObjectDefConstraint(closeBerriesID, avoidGold);

   int farBerriesID=rmCreateObjectDef("far berries");
   rmAddObjectDefItem(farBerriesID, "berry bush", 10, 4.0);
   rmSetObjectDefMinDistance(farBerriesID, 0.0);
   rmSetObjectDefMaxDistance(farBerriesID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(farBerriesID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(farBerriesID, avoidFood);
   rmAddObjectDefConstraint(farBerriesID, avoidGold);

   int closeBoarID=rmCreateObjectDef("close Boar");
   rmAddObjectDefItem(closeBoarID, "boar", 2, 4.0);
   rmSetObjectDefMinDistance(closeBoarID, 30.0);
   rmSetObjectDefMaxDistance(closeBoarID, 50.0);
   rmAddObjectDefConstraint(closeBoarID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeBoarID, avoidFood);

   int closeSettlementID=rmCreateObjectDef("close settlement");
   rmAddObjectDefItem(closeSettlementID, "Settlement", 1, 0.0);
   rmSetObjectDefMinDistance(closeSettlementID, 40.0);
   rmSetObjectDefMaxDistance(closeSettlementID, 60.0);
   rmAddObjectDefToClass(closeSettlementID, classPlayer);
   rmAddObjectDefConstraint(closeSettlementID, avoidSettlement);
   rmAddObjectDefConstraint(closeSettlementID, avoidTower2);
   rmAddObjectDefConstraint(closeSettlementID, edgeConstraint);
   rmAddObjectDefConstraint(closeSettlementID, shortAvoidGold);
   rmAddObjectDefConstraint(closeSettlementID, avoidImpassableLand);

   int mediumSettlementID=rmCreateObjectDef("medium settlement");
   rmAddObjectDefItem(mediumSettlementID, "Settlement", 1, 0.0);
   rmSetObjectDefMinDistance(mediumSettlementID, 40.0);
   rmSetObjectDefMaxDistance(mediumSettlementID, 60.0);
   rmAddObjectDefConstraint(mediumSettlementID, avoidSettlement);
   rmAddObjectDefConstraint(mediumSettlementID, edgeConstraint);
   rmAddObjectDefConstraint(mediumSettlementID, shortAvoidGold);
   rmAddObjectDefConstraint(mediumSettlementID, avoidImpassableLand);

   int farSettlementID=rmCreateObjectDef("far settlement");
   rmAddObjectDefItem(farSettlementID, "Settlement", 1, 0.0);
   rmSetObjectDefMinDistance(farSettlementID, 50.0);
   rmSetObjectDefMaxDistance(farSettlementID, 100.0);
   rmAddObjectDefConstraint(farSettlementID, avoidSettlement);
   rmAddObjectDefConstraint(farSettlementID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farSettlementID, edgeConstraint);
   rmAddObjectDefConstraint(farSettlementID, shortAvoidGold);
   rmAddObjectDefConstraint(farSettlementID, avoidImpassableLand);

   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "oak tree", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);

   int randomTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomTreeID, "oak tree", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);

   // This map will either use elephants, hippo, or deer as the extra huntable food.
   int classBonusHuntable=rmDefineClass("bonus huntable");
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 40.0);
   int avoidHuntable=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 20.0);
   int bonusHuntableID=rmCreateObjectDef("bonus huntable");
   float bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.5)   
      rmAddObjectDefItem(bonusHuntableID, "elephant", 2, 4.0);
   else if(bonusChance<0.75)
      rmAddObjectDefItem(bonusHuntableID, "hippo", 3, 6.0);
   else
      rmAddObjectDefItem(bonusHuntableID, "deer", 10, 8.0);
   rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
   rmSetObjectDefMinDistance(bonusHuntableID, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(bonusHuntableID, avoidHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);
   rmAddObjectDefConstraint(bonusHuntableID, avoidFood);

   // Pick another elephants, hippo, or deer as the extra huntable food.
   int bonusHuntableID2=rmCreateObjectDef("bonus huntable2");
   bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.5)   
      rmAddObjectDefItem(bonusHuntableID2, "elephant", 2, 4.0);
   else if(bonusChance<0.75)
      rmAddObjectDefItem(bonusHuntableID2, "hippo", 3, 6.0);
   else
      rmAddObjectDefItem(bonusHuntableID2, "deer", 10, 8.0);
   rmSetObjectDefMinDistance(bonusHuntableID2, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID2, rmXFractionToMeters(0.5));
   rmAddObjectDefToClass(bonusHuntableID2, classBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID2, farStartingSettleConstraint);
   rmAddObjectDefConstraint(bonusHuntableID2, avoidHuntable);
   rmAddObjectDefConstraint(bonusHuntableID2, avoidBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID2, avoidImpassableLand);
   rmAddObjectDefConstraint(bonusHuntableID2, avoidFood);


   // Relics
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 40.0);
   rmSetObjectDefMaxDistance(relicID, 100.0);
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, edgeConstraint);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 75.0));
   
   // Cheesy square placement of players.
   rmPlacePlayersSquare(0.3, 10.0, 10.0);

   // Build team areas.
   int teamClass=rmDefineClass("teamClass");
   int teamConstraint=rmCreateClassDistanceConstraint("team constraint", teamClass, 60.0);
   int teamEdgeConstraint=rmCreateBoxConstraint("team edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8), 0.01);
   float islandFraction=0.60/cNumberTeams;
   for(i=0; <cNumberTeams)
   {
      int teamID=rmCreateArea("team"+i);
      rmSetAreaSize(teamID, islandFraction, islandFraction);
      rmSetAreaLocation(teamID, 0.3, 0.5);
      rmSetAreaWarnFailure(teamID, false);
      rmSetAreaTerrainType(teamID, "GrassA");
      rmSetAreaMinBlobs(teamID, 1);
      rmSetAreaMaxBlobs(teamID, 5);
      rmSetAreaMinBlobDistance(teamID, 16.0);
      rmSetAreaMaxBlobDistance(teamID, 40.0);
      rmSetAreaCoherence(teamID, 0.0);
      rmAddAreaToClass(teamID, teamClass);
      rmSetAreaBaseHeight(teamID, 3.0);
      rmSetAreaHeightBlend(teamID, 2);
      rmAddAreaConstraint(teamID, teamConstraint);
      rmAddAreaConstraint(teamID, teamEdgeConstraint);
      rmSetAreaSmoothDistance(teamID, 10);
      rmSetAreaLocTeam(teamID, i);
   }
   rmBuildAllAreas();

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(4000);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i, rmAreaID("team"+rmGetPlayerTeam(i)));

      // Assign to the player.
      rmSetPlayerArea(i, id);

      // Set the size.
      rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);

      rmAddAreaToClass(id, classPlayer);

      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 5);
      rmSetAreaMinBlobDistance(id, 16.0);
      rmSetAreaMaxBlobDistance(id, 40.0);
      rmSetAreaCoherence(id, 0.0);

      // Add constraints.
      rmAddAreaConstraint(id, playerConstraint);

      // Set the location.
      rmSetAreaLocPlayer(id, i);

      // Set type.
      rmSetAreaTerrainType(id, "GrassDirt25");
   }

   // Build the areas.
   rmBuildAllAreas();

   for(i=1; <cNumberPlayers)
   {
      // Beautification sub area.
      int id2=rmCreateArea("Player inner"+i, rmAreaID("player"+i));
      rmSetAreaSize(id2, rmAreaTilesToFraction(400), rmAreaTilesToFraction(600));
      rmSetAreaLocPlayer(id2, i);
      rmSetAreaTerrainType(id2, "GrassDirt50");

      rmSetAreaMinBlobs(id2, 1);
      rmSetAreaMaxBlobs(id2, 5);
      rmSetAreaMinBlobDistance(id2, 5.0);
      rmSetAreaMaxBlobDistance(id2, 20.0);
      rmSetAreaCoherence(id2, 0.0);

      rmBuildArea(id2);
   }


   // Place starting settlements.
   // Close things....
   // TC
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   // Towers.
   for(j=0; <5)
      rmPlaceObjectDefPerPlayer(startingTowerID, true);

   // Straggler trees.
   for(j=0; <3)
      rmPlaceObjectDefPerPlayer(stragglerTreeID, false);

   // Settlement.
   //rmPlaceObjectDefPerPlayer(closeSettlementID, false);

   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false);

   // Chickens or berries.
   for(i=1; <cNumberPlayers)
   {
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmPlaceObjectDefAtLoc(closeChickensID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      else
         rmPlaceObjectDefAtLoc(closeBerriesID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }


   // Boar.
   rmPlaceObjectDefPerPlayer(closeBoarID, false);

   // Medium things....
   // Gold
   rmPlaceObjectDefPerPlayer(mediumGoldID, false);

   // Pigs
   for(j=0; <2)
   {
      for(i=1; <cNumberPlayers)
      rmPlaceObjectDefAtLoc(mediumPigsID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }   

   // Gazelle or deer.  Flip a coin.
   for(i=1; <cNumberPlayers)
   {
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmPlaceObjectDefAtLoc(mediumGazelleID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      else
         rmPlaceObjectDefAtLoc(mediumDeerID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

      
   // Far things.

   // Settlements.
   for(j=0; <2)
      rmPlaceObjectDefPerPlayer(farSettlementID, false);

   // Gold.
   for(j=0; <3)
      rmPlaceObjectDefPerPlayer(farGoldID, false);

   // Relics.
   rmPlaceObjectDefPerPlayer(relicID, false);

   // Bonus huntable.
   for(j=0; <cNumberNonGaiaPlayers)
      rmPlaceObjectDefAtLoc(bonusHuntableID, 0, 0.5, 0.5);

   // Pigs.
   for(j=0; <2)
   {
      for(i=1; <cNumberPlayers)
      rmPlaceObjectDefAtLoc(farPigsID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Berries.
   int numBerries=cNumberPlayers/2;
   for(j=0; <numBerries)
      rmPlaceObjectDefAtLoc(farBerriesID, 0, 0.5, 0.5);

   // Bears.
   for(j=0; <2)
      rmPlaceObjectDefPerPlayer(farBearsID, false);

   // Crocs.
   for(j=0; <2)
      rmPlaceObjectDefPerPlayer(farCrocsID, false);

   // Second bonus huntable stuff.
   for(j=0; <cNumberNonGaiaPlayers)
      rmPlaceObjectDefAtLoc(bonusHuntableID2, 0, 0.5, 0.5);



/*(
   // Elev hack.
   int numTries=40*cNumberNonGaiaPlayers;
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);
   for(i=0; <numTries)
   {
      int elevID=rmCreateArea("elev hack"+i, centerID);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, shortAvoidImpassableLand);
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmSetAreaTerrainType(elevID, "snowA");
      rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 4.0));

      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 5);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 40.0);
      rmSetAreaCoherence(elevID, 0.0);


      rmBuildArea(elevID);
   }
*/

   // Forest hack.
   int classForest=rmDefineClass("forest");
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 15.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int count=0;
   int numTries=8*cNumberNonGaiaPlayers;
   for(i=0; <numTries)
   {
      int forestID=rmCreateArea("forest hack"+i);
      rmSetAreaSize(forestID, rmAreaTilesToFraction(25), rmAreaTilesToFraction(100));
      rmSetAreaWarnFailure(forestID, false);
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmSetAreaForestType(forestID, "oak forest");
      else
         rmSetAreaForestType(forestID, "pine forest");
      rmAddAreaConstraint(forestID, forestSettleConstraint);
      rmAddAreaConstraint(forestID, forestObjConstraint);
      rmAddAreaConstraint(forestID, forestConstraint);
      rmAddAreaConstraint(forestID, shortAvoidImpassableLand);
      rmAddAreaToClass(forestID, classForest);
      
      rmSetAreaMinBlobs(forestID, 1);
      rmSetAreaMaxBlobs(forestID, 5);
      rmSetAreaMinBlobDistance(forestID, 16.0);
      rmSetAreaMaxBlobDistance(forestID, 40.0);
      rmSetAreaCoherence(forestID, 0.0);

      // Hill trees?
      if(rmRandFloat(0.0, 1.0)<0.2)
         rmSetAreaBaseHeight(forestID, rmRandFloat(3.0, 4.0));

      bool ok=rmBuildArea(forestID);
   }

   // Fish hack
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 10.0);
   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "fish - mahi", 1, 0.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, rmCreateTerrainDistanceConstraint("fish land", "land", true, 10.0));
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);

   // Random trees.
   int numRandomTrees=20*cNumberNonGaiaPlayers;
   for(i=0; <numRandomTrees)
      rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5);
   
}  
